﻿#include-once
#include <AutoItConstants.au3>
#include <debug.au3>
#include <InetConstants.au3>
#include <MsgBoxConstants.au3>
#include <StringConstants.au3>

#Region CertUtil.au3 - UDF Header
; #INDEX# ========================================================================
; Title .........: CertUtil.au3
; AutoIt Version : 3.3.10.2++
; Language ......: English
; Description ...: A collection of Function for use with an CertUtil.exe
; Author ........: mLipok
; Modified ......:
; Thanks to .....: Skysnake, tungpheng
; Support topic..: https://www.autoitscript.com/forum/topic/184937-certutil-udf-support-topic/
; Download ......: https://www.autoitscript.com/forum/files/file/421-certutil-udf/
; Date ..........: 2018/03/26
; Version .......: 0.0.3 BETA - Work in progress
; ================================================================================; ================================================================================

#cs
	2016/10/08
	.	First Released Version - mLipok

	2016/10/09
	.	Added: comments in example - mLipok

	2018/03/26
	.	Added: Function: _CertUtil_ConvertToBinary() - mLipok
	.	Added: Function: _CertUtil_Dump() - mLipok
	.	Added: Enums: $CUTIL_EXT_PARAM1, $CUTIL_EXT_PARAM2, $CUTIL_EXT_PARAM3 - mLipok
	.	Added: Enum: $CUTIL_ERR_FILE_EXISTS - mLipok

	@LAST
	@TODO - MANY THINGS :)

	;~ https://technet.microsoft.com/en-us/library/cc771320.aspx
	;~ https://www.autoitscript.com/forum/topic/9358-examples-of-comspec-formatting/
	;~ http://certificate.fyicenter.com/685_Microsoft_certutil-user_Certificate_Store_Locations.html
	;~ https://technet.microsoft.com/en-us/library/cc732443(v=ws.11).aspx#BKMK_Store
	;~ https://social.technet.microsoft.com/Forums/office/en-US/74735b08-0764-47a6-b454-56a8ce51d6db/how-to-viewexport-a-list-of-certificate-using-certutil-command-restrict-by-certificate-effective?forum=winserversecurity
	;~ https://blogs.technet.microsoft.com/pki/2008/10/03/disposition-values-for-certutil-view-restrict-and-some-creative-samples/

#ce
#EndRegion CertUtil.au3 - UDF Header

Global Enum _
		$CUTIL_ERR_SUCCESS, _
		$CUTIL_ERR_GENERAL, _
		$CUTIL_ERR_FILE_EXISTS, _
		$CUTIL_ERR_COUNTER

Global Enum _
		$CUTIL_EXT_DEFAULT, _
		$CUTIL_EXT_PARAM1, _
		$CUTIL_EXT_PARAM2, _
		$CUTIL_EXT_PARAM3, _
		$CUTIL_EXT_VERIFY_NOTFOUND, _
		$CUTIL_EXT_VERIFY_UNTRUSTEDCA, _
		$CUTIL_EXT_ADD_ALREADYINSTORE, _
		$CUTIL_EXT_COUNTER

Global Enum _
		$CUTIL_RET_FAILURE, _
		$CUTIL_RET_SUCCESS, _
		$CUTIL_RET_COUNTER

Global $__g_bCertTestingExample = False

Func _CertUtil_verifystore($sStore, $sCommonName)
	Local $sResult = __CertUtil_RunWrapper('-verifystore', $sStore, $sCommonName)
	If StringRegExp($sResult, '(?i)CertUtil: -verifystore command FAILED:.*?NTE_NOT_FOUND', $STR_REGEXPMATCH) Then
		; CertUtil: -verifystore command FAILED: 0x80090011 (-2146893807 NTE_NOT_FOUND)
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_VERIFY_NOTFOUND, 0)
	ElseIf StringRegExp($sResult, '(?i)CertUtil: -verifystore command FAILED:.*?CERT_E_UNTRUSTEDCA', $STR_REGEXPMATCH) Then
		; CertUtil: -verifystore command FAILED: 0x800b0112 (-2146762478 CERT_E_UNTRUSTEDCA)
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_VERIFY_UNTRUSTEDCA, 0)
	EndIf
	Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_DEFAULT, $CUTIL_RET_SUCCESS)
EndFunc   ;==>_CertUtil_verifystore

Func _CertUtil_delstore($sStore, $sCommonName)
	Local $sResult = __CertUtil_RunWrapper('-delstore', $sStore, $sCommonName)

	; CertUtil: -delstore command completed successfully.
	If Not StringInStr($sResult, 'CertUtil: -delstore command completed successfully.') Then
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_DEFAULT, 0)
	EndIf

	Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_DEFAULT, $CUTIL_RET_SUCCESS)

EndFunc   ;==>_CertUtil_delstore

Func _CertUtil_ImportPFX()
	; @TODO Work in progress
	Local $sResult = __CertUtil_RunWrapper('-f -user -p "Th1sIsN0tAR34lP4ssw0rd!" -importpfx "C:\Full\Path\To\Certificate.pfx" NoRoot', '', '')

	; CertUtil: -delstore command completed successfully.
	If Not StringInStr($sResult, 'CertUtil: -delstore command completed successfully.') Then
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_DEFAULT, 0)
	EndIf

	Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_DEFAULT, $CUTIL_RET_SUCCESS)

EndFunc   ;==>_CertUtil_ImportPFX

Func _CertUtil_Dump($sCert1_FileFullPath)
	; https://support.microsoft.com/en-us/help/216830/how-to-view-the-contents-of-a-certificate
	Local $sResult = __CertUtil_RunWrapper('-dump', $sCert1_FileFullPath)
	If Not StringInStr($sResult, 'CertUtil: -dump command completed successfully.') Then
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_DEFAULT, 0)
	EndIf

	Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_DEFAULT, $sResult)

EndFunc   ;==>_CertUtil_ConvertToBinary


Func _CertUtil_ConvertToBinary($sCert1_FileFullPath, $sCert2_FileFullPath = Default)
	; https://support.microsoft.com/en-us/help/216830/how-to-view-the-contents-of-a-certificate
	If $sCert2_FileFullPath = Default Then $sCert2_FileFullPath = $sCert1_FileFullPath & '.cer'
	Local $sResult = __CertUtil_RunWrapper('-decode', $sCert1_FileFullPath, $sCert2_FileFullPath)
	If StringInStr($sResult, '(WIN32: 80 ERROR_FILE_EXISTS)') Then
		Return SetError($CUTIL_ERR_FILE_EXISTS, $CUTIL_EXT_PARAM2, 0)
	ElseIf Not StringInStr($sResult, 'CertUtil: -decode command completed successfully.') Then
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_DEFAULT, 0)
	EndIf

	Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_DEFAULT, $CUTIL_RET_SUCCESS)

EndFunc   ;==>_CertUtil_ConvertToBinary

Func _CertUtil_addstore($sStore, $sCert_FileFullPath)
	Local $sResult = __CertUtil_RunWrapper('-addstore', $sStore, $sCert_FileFullPath)
	If Not StringInStr($sResult, 'CertUtil: -addstore command completed successfully.') Then
		Return SetError($CUTIL_ERR_GENERAL, $CUTIL_EXT_DEFAULT, 0)
	EndIf

	If StringRegExp($sResult, '(?i)Certificate ".*?" already in store.', $STR_REGEXPMATCH) Then
		; Certificate "CERTUM QCA" already in store.
		Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_ADD_ALREADYINSTORE, $CUTIL_RET_SUCCESS)
	EndIf

	Return SetError($CUTIL_ERR_SUCCESS, $CUTIL_EXT_DEFAULT, $CUTIL_RET_SUCCESS)

EndFunc   ;==>_CertUtil_addstore

Func __CertUtil_RunWrapper($sParam0_Command, $sParam1_Store, $sParam2 = "")
	Local $sCommand = "certutil.exe " & $sParam0_Command & " " & $sParam1_Store & " " & $sParam2
	If $__g_bCertTestingExample Then _DebugOut( _
			'====================================================================' & @CRLF & _
			'Command: ' & $sCommand)
	Local $iPID = Run(@ComSpec & " /c " & $sCommand, @SystemDir, @SW_HIDE, $STDERR_CHILD + $STDOUT_CHILD)
	Local $sOutput = "Stdout Read:" & @CRLF
	While 1
		$sOutput &= StdoutRead($iPID)
		; Exit the loop if the process closes or StdoutRead returns an error.
		If @error Then ExitLoop
	WEnd

	Local $sOutputErr = ""
	While 1
		$sOutputErr &= StderrRead($iPID)
		; Exit the loop if the process closes or StderrRead returns an error.
		If @error Then ExitLoop
	WEnd
	If Not $sOutputErr = "" Then
		$sOutput = $sOutput & @CRLF & "Stderr Read:" & @CRLF & $sOutputErr
	EndIf

	$sOutput = $sOutput & _
			'====================================================================' & @CRLF & _
			''

	If $__g_bCertTestingExample Then _DebugOut($sOutput)
	Return $sOutput
EndFunc   ;==>__CertUtil_RunWrapper

Func _CertUtil_RunCertMgr()
	ShellExecute(@SystemDir & '\certmgr.msc')
EndFunc   ;==>_CertUtil_RunCertMgr
