﻿#include <Array.au3>
#include <File.au3>
#include <String.au3>
#include "CertUtil.au3"

$__g_bCertTestingExample = True

_CertUtil_Example_Dump()
;~ _CertUtil_Example()

Func _CertUtil_Example_Dump()
	Local $sFile1 = 'C:\1\cert.txt'
	Local $sFile2 = 'C:\1\cert2.cer'

	Local $sXML_File = FileOpenDialog('Choose XML File', @ScriptDir, 'XML Files (*.xml)')
	If @error Then Return SetError(@error, @extended, 0)

	ConsoleWrite("! $sXML_File = " & $sXML_File & @CRLF)
	Local $hFile = FileOpen($sXML_File, $FO_READ + $FO_UTF8_NOBOM)
	Local $sFileContent = FileRead($hFile)
	FileClose($hFile)

	Local $aX509Certificate = _StringBetween($sFileContent, '<X509Certificate>', '</X509Certificate>')
	If @error Then MsgBox($MB_ICONERROR, '_StringBetween', '@error = ' & @error & @CRLF & '@extended = ' & @extended)

	Local $sX509Certificate = $aX509Certificate[0]

	Local $hFile = FileOpen($sFile1, $FO_OVERWRITE + $FO_CREATEPATH + $FO_BINARY )
	FileWrite($hFile, '-----BEGIN CERTIFICATE-----' & @CRLF & $sX509Certificate & @CRLF & '-----END CERTIFICATE-----')
	FileClose($hFile)

	_CertUtil_ConvertToBinary($sFile1, $sFile2)
	If @error Then MsgBox($MB_ICONERROR, '_CertUtil_ConvertToBinary', '@error = ' & @error & @CRLF & '@extended = ' & @extended)

	Local $sResult = _CertUtil_Dump($sFile2)
	If @error Then MsgBox($MB_ICONERROR, '_CertUtil_Dump', '@error = ' & @error & @CRLF & '@extended = ' & @extended)

	MsgBox(0, '$sResult', $sResult)

	ClipPut($sResult)

EndFunc   ;==>_Example

Func _CertUtil_Example()
	If $__g_bCertTestingExample Then _DebugSetup()

;~ 	_CertUtil_ImportPFX()
;~ 	Exit

	; Get list of "Polish Certyfication Centers" form "National Certyfication Center"
	Local $dData = InetRead('https://www.nccert.pl/zaswiadczenia.htm', $INET_FORCERELOAD)
	Local $iBytesRead = @extended

	; Convert the ANSI compatible binary string back into a string.
	Local $sData = BinaryToString($dData)

	; Find only QCQ *.crt or *.cer files
	Local $aFiles = StringRegExp($sData, '(?is)href="files/(.{1,20}QCA.{0,20}.c[ert]{2})"', $STR_REGEXPARRAYGLOBALMATCH)

;~ 	_ArrayDisplay($aFiles,'$aFiles' )

	Local $sFileToDownload = ''
	Local $sFileFullPath_LocalDest = ''

	For $iFile_idx = 0 To UBound($aFiles) - 1
		$sFileToDownload = $aFiles[$iFile_idx]
		; Download QCQ *.crt or *.cer file
		_CertUtil_DownloadCertFile($sFileToDownload)
		$sFileFullPath_LocalDest = @ScriptDir & '\' & StringReplace($sFileToDownload, '%20', ' ')
		; Add downloaded QCA file to the "Windows Certification Storage"
		_CertUtil_addstore('-enterprise CA', $sFileFullPath_LocalDest)
	Next

EndFunc   ;==>_CertUtil_Example

Func _CertUtil_DownloadCertFile($sFileToDownload)

	Local $sURL_nccert = 'https://www.nccert.pl/files/'

	$sFileToDownload = StringReplace($sFileToDownload, '%20', ' ')
	Local $sHTTP_Source = $sFileToDownload
	If Not StringInStr($sFileToDownload, '://') Then $sHTTP_Source = $sURL_nccert & $sHTTP_Source

	Local $sFileFullPath_LocalDest = @ScriptDir & '\' & StringReplace($sFileToDownload, '%20', ' ')
	Local $sFileFullPath_LocalDest_Size = FileGetSize($sFileFullPath_LocalDest)

;~ 	ConsoleWrite("! $sHTTP_Source=" & $sHTTP_Source & @CRLF)

	Local $sHTTP_Source_size = InetGetSize($sHTTP_Source, $INET_FORCERELOAD + $INET_BINARYTRANSFER)
	If $sHTTP_Source_size <> 0 And $sHTTP_Source_size <> $sFileFullPath_LocalDest_Size Then
		If $__g_bCertTestingExample Then _DebugOut('Downloading: ' & $sHTTP_Source)
		Local $hDownload = InetGet( _
				$sHTTP_Source, _
				$sFileFullPath_LocalDest, _
				$INET_FORCERELOAD, $INET_DOWNLOADBACKGROUND)
		If @error Then
			InetClose($hDownload)
			Return
		EndIf

		Local $aData, $s_old_data = ''
		Do
			$aData = InetGetInfo($hDownload) ; Get all information.
			If IsArray($aData) = 0 Then ContinueLoop
			If $aData[0] <> $s_old_data And $__g_bCertTestingExample Then _DebugOut($sFileToDownload & ' >> Downloaded:_______' & $aData[0] & '_______z_______' & $aData[1])
			$s_old_data = $aData[0]
		Until $aData[2] = True
		InetClose($hDownload) ; Close the handle to release resourcs.
	EndIf
EndFunc   ;==>_CertUtil_DownloadCertFile
